import os
import sys
sys.path.insert(0, '..')
from model.keras.yolo import YOLO
from model.openCv.OpenCv import OpenCV
from timeit import default_timer as timer
import numpy as np
import json


def load(model_name):
    if model_name.startswith("keras"):
        return YOLO()
    elif model_name.startswith("opencv"):
        version = model_name.split('_',1)[1]
        return OpenCV(version)
    else:
        raise ValueError(f"Model {model_name} not found")

#json resultados
performance = {}
data = []
videopath = 'people3.mp4'

def ejecutarModelo(modelName,videopath):
    model = load(modelName)
    print("empezo ejecucion de modelo")
    start = timer()
    resultado = model.analyze_video(videopath) #path de video
    end = timer()
    meanPersons = np.mean(resultado)
    tiempoTotal = end - start
    FPS_Promedio = ((len(resultado))/(end-start))
    dataJson = {
        'Modelo': modelName,
        'PersonasPromedio': meanPersons,
        'tiempoTotal': tiempoTotal,
        'FPS_Promedio': FPS_Promedio
    }
    data.append(dataJson)
    return dataJson

#OpenCV tiny
modelData = ejecutarModelo('opencv_tiny',videopath)
print(modelData)

#OpenCV 320
modelData = ejecutarModelo('opencv_320',videopath)
print(modelData)

#OpenCV 416
modelData = ejecutarModelo('opencv_416',videopath)
print(modelData)

#OpenCV 608
modelData = ejecutarModelo('opencv_608',videopath)
print(modelData)

#Keras
modelData = ejecutarModelo('keras_default',videopath)
print(modelData)


performance['DatosVideo'] = data
with open('{}.json'.format(videopath), 'w') as file:
    json.dump(performance, file, indent=4)
print(performance)







import os
import pathlib
import sys
#os.path.insert(0, '../model')
#from base import BaseModel
sys.path.insert(0, '../model/keras')
from yolo import YOLO
from timeit import default_timer as timer
import numpy as np
#archivo para pruebas

#os.system(f'python ../model/keras/yolo.py people3.mp4')
#os.system(f'python ../model/openCv/OpenCv.py people3.mp4 yolov3-320.cfg yolov3.weights')

kerasModel = load("yolo3")
print("empezo ejecucion de modelo")
start = timer()
resultado = kerasModel.analyze_video('people3.mp4')
end = timer()
meanPersons = np.mean(resultado)
print('Rendimiento ----------')
print('Personas encontradas en promedio por frame : ',meanPersons) 
print("tiempo total : ",end-start)
print("FPS promedio : ",((len(resultado))/(end-start)))
print('-----------------------')


'''funcion para imprimir json (se llama con -json_write(data))
        def json_write(data):
            #guardo datos en un archivo json
            with open('{}.json'.format(video_path), 'w') as file:
                json.dump(data, file, indent=4)

        def json_write(data):
            """guardo datos en un archivo json"""
            with open('data.json', 'w') as file:
                json.dump(data, file, indent=4)'''


# -*- coding: utf-8 -*-

import sys
import os
import json
import numpy as np
import tensorflow.compat.v1.keras.backend as K
import tensorflow as tf
tf.compat.v1.disable_eager_execution()
from timeit import default_timer as timer
from keras.models import load_model
from keras.layers import Input
#from keras.utils import multi_gpu_model
from PIL import Image, ImageFont, ImageDraw
#print(sys.path)
#sys.path.insert(0, './yolo3')
#print("syspath modificado",sys.path)
from ..base import BaseModel 
from model.keras.yolo3.utils import letterbox_image
from model.keras.yolo3.model import yolo_eval, yolo_body, tiny_yolo_body
import os


relative_path = os.path.dirname(os.path.relpath(__file__))


class YOLO(BaseModel):
    _defaults = {
        "model_path": '/../cfg/yolo.h5',
        "anchors_path": '/../cfg/yolo_anchors.txt',
        "classes_path": '/../cfg/coco_classes.txt',
        "score" : 0.3,
        "iou" : 0.45,
        "model_image_size" : (416, 416),
        "gpu_num" : 1,
    }

    @classmethod
    def get_defaults(cls, n):
        if n in cls._defaults:
            return cls._defaults[n]
        else:
            return "Unrecognized attribute name '" + n + "'"

    def __init__(self):
        self.__dict__.update(self._defaults) # set up default values
        self.class_names = self._get_class()
        self.anchors = self._get_anchors()
        self.sess = K.get_session()
        self.boxes, self.scores, self.classes = self.generate()


    def _get_class(self):
        classes_path = relative_path+str(self.classes_path)
        with open(classes_path) as f:
            class_names = f.readlines()
        class_names = [c.strip() for c in class_names]
        return class_names

    def _get_anchors(self):
        anchors_path = relative_path+str(self.anchors_path)
        with open(anchors_path) as f:
            anchors = f.readline()
        anchors = [float(x) for x in anchors.split(',')]
        return np.array(anchors).reshape(-1, 2)

    def generate(self):
        model_path = relative_path+str(self.model_path)
        assert model_path.endswith('.h5'), 'Keras model or weights must be a .h5 file.'

        # Load model, or construct model and load weights.
        num_anchors = len(self.anchors)
        num_classes = len(self.class_names)
        is_tiny_version = num_anchors==6 # default setting
        try:
            self.yolo_model = load_model(model_path, compile=False)
        except:
            self.yolo_model = tiny_yolo_body(Input(shape=(None,None,3)), num_anchors//2, num_classes) \
                if is_tiny_version else yolo_body(Input(shape=(None,None,3)), num_anchors//3, num_classes)
            self.yolo_model.load_weights(self.model_path) # make sure model, anchors and classes match
        else:
            assert self.yolo_model.layers[-1].output_shape[-1] == \
                num_anchors/len(self.yolo_model.output) * (num_classes + 5), \
                'Mismatch between model and given anchor and class sizes'

        print('{} model, anchors, and classes loaded.'.format(model_path))
        
        # Generate output tensor targets for filtered bounding boxes.
        # Lo guardamos por las dudas (conf GPU)
        self.input_image_shape = K.placeholder(shape=(2, ))
        '''if self.gpu_num>=2:
            self.yolo_model = multi_gpu_model(self.yolo_model, gpus=self.gpu_num)'''
        boxes, scores, classes = yolo_eval(self.yolo_model.output, self.anchors,
                len(self.class_names), self.input_image_shape,
                score_threshold=self.score, iou_threshold=self.iou)
        return boxes, scores, classes
    
    def analyze_frame(self, image):
        if self.model_image_size != (None, None):
            assert self.model_image_size[0]%32 == 0, 'Multiples of 32 required'
            assert self.model_image_size[1]%32 == 0, 'Multiples of 32 required'
            boxed_image = letterbox_image(image, tuple(reversed(self.model_image_size)))
        else:
            new_image_size = (image.width - (image.width % 32),
                              image.height - (image.height % 32))
            boxed_image = letterbox_image(image, new_image_size)
        image_data = np.array(boxed_image, dtype='float32')
        image_data /= 255.
        image_data = np.expand_dims(image_data, 0)  # Add batch dimension.
        '''filtrar cajas con puntajes mayores a 0.3'''

        out_boxes, out_scores, out_classes = self.sess.run(
            [self.boxes, self.scores, self.classes],
            feed_dict={
                self.yolo_model.input: image_data,
                self.input_image_shape: [image.size[1], image.size[0]],
                K.learning_phase(): 0
            })
        cantNoPersona = 0
        for objeto in out_classes:
            if objeto != 0:
                cantNoPersona = cantNoPersona + 1
        #data['list'].append((len(out_boxes)-cantNoPersona))
        return (len(out_boxes)-cantNoPersona)


    def close_session(self):
        self.sess.close()

    def analyze_video(self, video_path):
        import cv2
        vid = cv2.VideoCapture(video_path)
        if not vid.isOpened():
            raise IOError("Couldn't open webcam or video")
        video_FourCC     = cv2.VideoWriter_fourcc(*'XVID')
        video_fps       = vid.get(cv2.CAP_PROP_FPS)
        video_size      = (int(vid.get(cv2.CAP_PROP_FRAME_WIDTH)),
                            int(vid.get(cv2.CAP_PROP_FRAME_HEIGHT)))
        results = []
        while True:
            print(".")
            return_value, frame = vid.read()
            if not return_value:
                break
            image = Image.fromarray(frame)
            results.append(self.analyze_frame(image))
        return results




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pip install twine

from setuptools import find_packages, setup, Command
from shutil import rmtree
import sys
import os
import io
from pathlib import Path

# Package meta-data.
NAME = 'cctv-analysis'
URL="https://github.com/midusi/cctv_analysis"


here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
VERSION = "0.0.1"


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        
        dist_path=Path(here)/'dist'
        if dist_path.exists():
            self.status('Removing previous builds…')
            rmtree(dist_path)


        self.status('Building Source and Wheel (universal) distribution…')
        os.system(
            '{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(VERSION))
        os.system('git push --tags')

        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=VERSION,
    description='A ready-to-use server to analyse cctv feeds. Pip install, boot up server, and make requests to analyse videos.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Juan Tomás Barrón - Gustavo Risso - Facundo Quiroga',
    author_email='fquiroga@lidi.info.unlp.edu.ar',
    python_requires='>=3.6.0',
    url=URL,
    project_urls={
        "Bug Tracker": URL+"/issues",
        "Documentation": URL,
        "Source Code": URL,
    },
    # dont include the performance folder 
    packages=find_packages(exclude=('performance')),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],

    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    install_requires= [ "flask","gdown","numpy","python-opencv","tensorflow-gpu","keras","pillow","matplotlib","tqdm"],
    include_package_data=True,
    license='GNU Affero General Public License v3 or later (AGPLv3+)',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        # 'Programming Language :: Python :: Implementation :: PyPy',
        # 'Programming Language :: Python :: 3 :: Only',
        "License :: OSI Approved :: Python Software Foundation License",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
    ],
    # $ setup.py publish support.
    cmdclass={
        'upload': UploadCommand,
    },
)
